import org.bouncycastle.crypto.digests.SHA256Digest;
import org.bouncycastle.crypto.generators.PKCS5S2ParametersGenerator;
import org.bouncycastle.crypto.params.KeyParameter;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.Charset;
import java.security.GeneralSecurityException;
import java.security.spec.KeySpec;


public class SimpleEncryptor {

    public static final String DEFAULT_CIPHER_ALGORITHM = "AES/CBC/PKCS5Padding";

    private static final Charset characterSetUtf8 = Charset.forName("UTF-8");

    private static final Charset characterSetAscii = Charset.forName("US-ASCII");

    private static final byte[] IV = {10, 2, 3, -4, 20, 23, 47, -38, 27, -22, 14, -20, -22, 37, 36, 54};
    private static final byte[] KeyValue = new byte[]{82, 26, -72, 65, -36, 122, -35, 0, -87, 41 - 39, 42, 27, 44, 6 - 97, 86, -36, 124, 115, 82, -97, 115, -85, 40, 107, -17, 120, -10, -107, 13, -48};
    private Cipher encryptor;
    private Cipher decryptor;

    public SimpleEncryptor(byte[] salt) {
        this(DEFAULT_CIPHER_ALGORITHM, IV, KeyValue, salt);
    }

    public SimpleEncryptor(byte[] IV, byte[] keyValue, byte[] salt) {
        this(DEFAULT_CIPHER_ALGORITHM, IV, keyValue, salt);
    }

    public SimpleEncryptor(String cipherAlgorithm, byte[] IV, byte[] keyValue, byte[] salt) {
        try {


            //PBKDF2WithHmacSHA256
            final PKCS5S2ParametersGenerator gen = new PKCS5S2ParametersGenerator(new SHA256Digest());
            gen.init(keyValue, salt, 4096);
            final byte[] dk = ((KeyParameter) gen.generateDerivedParameters(256)).getKey();

            SecretKey secret = new SecretKeySpec(dk, "AES");
            encryptor = Cipher.getInstance(cipherAlgorithm);
            encryptor.init(Cipher.ENCRYPT_MODE, secret, new IvParameterSpec(IV));
            decryptor = Cipher.getInstance(cipherAlgorithm);
            decryptor.init(Cipher.DECRYPT_MODE, secret, new IvParameterSpec(IV));
        } catch (GeneralSecurityException e) {
            throw new RuntimeException(e);
        }
    }

    public byte[] encrypt(byte[] source) {
        if (source == null) {
            return null;
        }
        try {
            return encryptor.doFinal(source);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public byte[] decrypt(byte[] source) {
        if (source == null) {
            return null;
        }
        try {
            return decryptor.doFinal(source);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public String encrypt(String source) {
        if (source == null) {
            return null;
        }
        try {
            byte[] payload = encrypt(source.getBytes(characterSetAscii));
            if (payload == null) {
                return null;
            }
            return new String(Base64.encode(payload));
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public String decrypt(String source) {
        if (source == null) {
            return null;
        }
        try {

            byte[] payload = decrypt(Base64.decode(source));
            if (payload == null) {
                return null;
            }
            return new String(payload, characterSetUtf8);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

}
